import socket
import json
import threading
import time

def get_local_ip():
    host_name = socket.gethostname()
    host_ip = socket.gethostbyname(host_name)
    return host_ip


def get_mac_addr():
    import uuid
    mac = hex(uuid.getnode())[2:]
    mac = f"{mac[0:2]}:{mac[2:4]}:{mac[4:6]}:{mac[6:8]}:{mac[8:10]}:{mac[10:12]}"
    return mac


def emptyCallback(*args, **kwargs):
    pass


def default_discovery_callback(cls):
    cls.reply()
    print(cls)



import socket
import json
import threading
import time

from .utils import *
from .message import *

class DiscoveryClient:
    def __init__(self, host="255.255.255.255", port=12892, buffer_size=1024, scheme="_discovery.default",
                 device_name=None):
        self.host = host
        self.port = port
        self.buffer_size = buffer_size
        self.scheme = scheme
        self.device_name = device_name
        if device_name is None:
            self.device_name = socket.gethostname()

        self.sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        self.sock.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)

        self.running = False

        self.results = []

    def form_msg(self, data):
        return json.dumps({
            "proto": "_cosmo.discovery",
            "request": "discovery.find",
            "scheme": self.scheme,
            "host": {"ip": get_local_ip(), "port": self.port, "mask": self.host, "mac": get_mac_addr(),
                     "hostname": socket.gethostname()},
            "data": data
        })

    def push(self, data=None):
        if data is None:
            data = {}
        self.sock.sendto(self.form_msg(data).encode("UTF-8"), (self.host, self.port))

    def listen(self):
        while self.running:
            msg, addr = self.sock.recvfrom(self.buffer_size)
            self._handle_msg(msg, addr)

    def discovery(self, timeout=5, data=None):
        self.running = True
        thd = threading.Thread(target=self.listen,daemon=True)
        thd.start()
        self.push(data)
        time.sleep(timeout)
        self.running = False

        res = self.results.copy()
        self.results = []
        return res

    def _handle_msg(self, msg, addr):
        try:
            data = json.loads(msg.decode("UTF-8"))
        except json.JSONDecodeError:
            return None

        if not data["proto"] == "_cosmo.discovery":  # Check Proto Var
            return None

        if not data["scheme"] == self.scheme:  # Check Scheme Name
            return None

        if data["request"] == "discovery.reply":
            cls = DiscoveryReplyMessage(self, data, addr)
            self.results.append(DiscoveryResult(cls))
            return cls



import socket
import json
import threading
import time

from .utils import *
from .message import *

class DiscoveryServer:
    def __init__(self, host="", port=12892, buffer_size=1024, scheme="_discovery._cosmo.default"):
        self.host = host
        self.port = port
        self.buffer_size = buffer_size
        self.scheme = scheme

        self.sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        self.running = False

        self.discovery_callback_func = default_discovery_callback

    def listen(self, threaded=False):
        if threaded:
            thd = threading.Thread(target=self._run,daemon=True)
            thd.start()
        else:
            self._run()

    def _run(self):
        self.sock.bind((self.host, self.port))
        self.running = True
        self._handle()

    def discovery_callback(self, func):
        self.discovery_callback_func = func

    def _handle(self):
        while self.running:
            msg, addr = self.sock.recvfrom(self.buffer_size)

            self._handle_msg(msg, addr)

    def _handle_msg(self, msg, addr):
        try:
            data = json.loads(msg.decode("UTF-8"))
        except json.JSONDecodeError:
            return None

        if not data["proto"] == "_cosmo.discovery":  # Check Proto Var
            return None

        if not data["scheme"] == self.scheme:  # Check Scheme Name
            return None

        if data["request"] == "discovery.find":
            cls = DiscoveryRequestMessage(self, data, addr)
            self.discovery_callback_func(cls)
            return cls

    def close(self):
        self.running = False
        
        
class DiscoveryResult:
    def __init__(self, msg):
        self.msg = msg

        self.data = msg.data
        self.ip = msg.host_ip
        self.mac = msg.host_mac
        self.name = msg.host_name

    def __repr__(self):
        return str({"data": self.data, "ip":self.ip, "mac":self.mac, "name":self.name})



import socket
import json
import threading
import time


class DiscoveryMessage:
    def __init__(self, master, data, addr):
        self.master = master
        self.addr = addr
        self.raw = data
        self.proto = data["proto"]
        self.request = data["request"]
        self.scheme = data["scheme"]
        self.host_raw = data["host"]
        self.host_ip = data["host"]["ip"]
        self.host_port = data["host"]["port"]
        self.host_mask = data["host"]["mask"]
        self.host_mac = data["host"]["mac"]
        self.host_name = data["host"]["hostname"]

        self.data = data["data"]

    def reply(self, msg):
        self.master.sock.sendto(msg.encode("UTF-8"), self.addr)

    def __repr__(self):
        return str(self.raw)


class DiscoveryRequestMessage(DiscoveryMessage):
    def __init__(self, master, data, addr):
        super().__init__(master, data, addr)

    def reply(self, value=True, data=None):
        if data is None:
            data = {}
        msg = self.form_msg(data)
        super().reply(msg)

    def form_msg(self, data):
        return json.dumps({
            "proto": "_cosmo.discovery",
            "request": "discovery.reply",
            "scheme": self.scheme,
            "host": {"ip": get_local_ip(), "port": self.master.port, "mask": self.master.host, "mac": get_mac_addr(),
                     "hostname": socket.gethostname()},
            "data": data
        })


class DiscoveryReplyMessage(DiscoveryMessage):
    def __init__(self, master, data, addr):
        super().__init__(master, data, addr)
